# The Netlogon Project
#
# auth.py - authentication services
#
# $Id: auth.py,v 1.9 2001/03/08 14:54:51 kent Exp $
#
# Copyright (C) 2000,2001 UNIT, Linkping University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import path
import os
import string
import sys
import signal
import time
import conf
import poplib
import imaplib
import log
import nis
import crypt

#
# Exceptions
#

class AuthError: pass
class AuthBadLogin(AuthError): pass  # The login as such is bad
class AuthSystemFailure(AuthError): pass # The system failed (not the users fault)

#
# Basklass fr alla autentiseringsklasser
#

class AuthBase:
    def __init__(self, c, code, lang):
        self._code = code
        self._name = c.get(code, "Name_" + lang)
        try:
            self._position = c.getint(code, "Position")
        except:
            self._position = 100
            
            
    def get_position_name_and_code_tuple(self):
        # Used by Auth.get_codes_and_names
        return (self._position, self._name, self._code)

#
# Autentiseringsklass FIXED:
#   En enda anvndare, fast lsenord. Fr testning
#

class AuthFixed(AuthBase):
    def __init__(self, c, code, lang):
        AuthBase.__init__(self, c, code, lang)
        self._user = c.get(code, "User")
        self._password = c.get(code, "Password")

    def authenticate(self, user, password):
        if not (user == self._user and password == self._password):
            raise AuthBadLogin

#
# Autentiseringsklass SSHMAILAUTH:
#   Gr ssh till en server. Tala om anvndarnamn och lsen. F svar.
#

class AuthSSHMailAuth(AuthBase):
    def __init__(self, c, code, lang):
        AuthBase.__init__(self, c, code, lang)
        self._server = c.get(code, "Server")
        self._port = c.getint_default(code, "Port", 22)
        self._timeout = c.getint_default(code, "Timeout", 20)
        self._user = c.get(code, "User")
        self._keyfile = c.get(code, "Keyfile")

    def authenticate(self, user, password):
        # Connect to the server
        try:
            # FIXME: fixa timeout (alarm funkade inte trivialt)
            cmd = "/usr/bin/ssh -p %d -i %s -l %s -o BatchMode=yes %s >/dev/null 2>&1" % \
                  (self._port, self._keyfile, self._user, self._server)
            f = os.popen(cmd, "w")
            f.write("%s\n%s\n" % (user, password))
            rc = f.close()
        except:
            log.log("AuthSSHMailAuth failure authenticating %s, raised exception" % user,
                    log.LOG_WARNING)
            raise AuthSystemFailure

        if rc is None:
            return
        elif rc == 256:
            raise AuthBadLogin
        else:
            log.log("AuthSSHMailAuth failure authenticating %s, return code %d" % (user, rc),
                    log.LOG_WARNING)
            raise AuthSystemFailure

#
# Autentiseringsklass POP3:
#   Frsk logga in med POP3 till en server.
#

class AuthPOP3(AuthBase):
    def __init__(self, c, code, lang):
        AuthBase.__init__(self, c, code, lang)
        self._server = c.get(code, "Server")

    def authenticate(self, user, password):
        # Connect to the server
        try:
            p = poplib.POP3(self._server)
            try:
                p.user(user)
                p.pass_(password)
            except:
                p.quit()
                raise AuthBadLogin
            p.quit()
        except:
            log.log("AuthPOP3 failure authenticating %s, raised exception" % user, log.LOG_WARNING)
            raise AuthSystemFailure

#
# Autentiseringsklass IMAP4:
#   Frsk logga in med IMAP4 till en server.
#

class AuthIMAP4(AuthBase):
    def __init__(self, c, code, lang):
        AuthBase.__init__(self, c, code, lang)
        self._server = c.get(code, "Server")

    def authenticate(self, user, password):
        # Connect to the server
        try:
            p = imaplib.IMAP4(self._server)
            try:
                p.login(user, password)
            except:
                raise AuthBadLogin
        except:
            log.log("AuthIMAP4 failure authenticating %s, raised exception" % user, log.LOG_WARNING)
            raise AuthSystemFailure
            
#
# Autentiseringsklass NIS:
#   Kontrollera lsenord mot NIS-mapp
#

class AuthNIS(AuthBase):
    def __init__(self, c, code, lang):
        AuthBase.__init__(self, c, code, lang)

    def authenticate(self, user, password):
        # Get NIS row
        try:
            try:
                row = nis.match(user, "passwd.byname")
            except nis.error:
                # Assume this is because the user does not exist
                raise AuthBadLogin
            
            # Check password
            fields = string.split(row, ":")
            cry_password = crypt.crypt(password, fields[1])
            if fields[1] <> cry_password:
                raise AuthBadLogin

        except AuthBadLogin:
            raise
        
        except:
            log.log("AuthNIS failure authenticating %s, raised exception" % user, log.LOG_WARNING)
            raise AuthSystemFailure
    
#
# Klass fr att hlla reda p alla metoder
#

_auth_dict = {"fixed": AuthFixed,
              "sshmailauth": AuthSSHMailAuth,
              "pop3": AuthPOP3,
              "imap4": AuthIMAP4,
              "nis": AuthNIS}

class Auth:
    def __init__(self, lang = "sv"):
        self.__auth_list = []
        self.__auth_dict = {}
        try:
            c = conf.Config("auth")

            for code in c.sections():
                try:
                    # Convert name to class, instantiate object, store it
                    auth_type = string.lower(c.get(code, "Type"))
                    auth_class = _auth_dict[auth_type]
                    auth_obj = auth_class(c, code, lang)
                    self.__auth_list.append(auth_obj)
                    self.__auth_dict[code] = auth_obj
                except:
                    pass
        except:
            pass
    
    def get_codes_and_names(self):
        # Return a list of tuples (code, name) for valid authentication methods
        tuples = map(lambda x: x.get_position_name_and_code_tuple(),
                     self.__auth_list)
        tuples.sort()
        return map(lambda x: (x[2],x[1]), tuples)

    def authenticate(self, logon_type, logon_user, logon_password):
        # Return true iff logon is successful
        try:
            obj = self.__auth_dict[logon_type]
        except:
            log.log("Auth failure: cannot find type %s (user %s)" % (logon_type, logon_user),
                    log.LOG_ALERT)
            raise AuthSystemFailure

        # Delegate
        return obj.authenticate(logon_user, logon_password)


