# The Netlogon Project
#
# conf.py - access to configuration files
#
# $Id: conf.py,v 1.9 2001/03/08 14:54:51 kent Exp $
#
# Copyright (C) 2000,2001 UNIT, Linkping University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import path

import ConfigParser
import os
import string
import ipaddr

#
# General protected interface to ConfigParser for our needs
# (returning empty / default when not existing)
#

class Config:
    def __init__(self, basename):
        self.__file = os.path.join(path.confdir, basename + ".conf")
        try:
            self.__CP = ConfigParser.ConfigParser()
            self.__CP.read(self.__file)
        except:
            self.__CP = None

    def get(self, section, option):
        return self.__CP.get(section, option)
        
    def get_default(self, section, option, default = None):
        try:
            return self.get(section, option)
        except:
            return default
        
    def getint(self, section, option):
        return self.__CP.getint(section, option)
        
    def getint_default(self, section, option, default = None):
        try:
            return self.getint(section, option)
        except:
            return default
        
    def getbool(self, section, option):
        txt = string.upper(self.__CP.get(section, option))
        if txt in ["YES", "ON", "TRUE"]:
            return 1
        else:
            return 0
        
    def getbool_default(self, section, option, default = None):
        try:
            return self.getbool(section, option)
        except:
            return default
        
    def sections(self):
        try:
            return self.__CP.sections()
        except:
            return []
        
            
#
# Accessing interfaces.conf
#

IF_SERVICE = 1
IF_DEFAULT = 2
IF_ACCESS = 3
_if_type_dict = {"service": IF_SERVICE,
                 "default": IF_DEFAULT,
                 "access": IF_ACCESS}


#
# Interfaces
#

class Interfaces:
    def __init__(self):
        # Read interfaces.conf
        self.conf = Config("interfaces")

        # Create interface objects
        self.__ifdict = {}
        for ifname in self.conf.sections():
            self.__ifdict[ifname] = Interface(ifname, self.conf)

    def __getitem__(self, ifname):
        # Indexing by name
        return self.__ifdict[ifname]

    def get_all(self):
        # Return a plain list of all interface objects
        return self.__ifdict.values()

    def get_all_of_type(self, iftype):
        # Return a plain list of all interface objects of a certain type
        return filter(lambda x, iftype=iftype: x.type == iftype, self.__ifdict.values())

    def get_default_gateway(self):
        # Return default gateway address
        return self.get_all_of_type(IF_DEFAULT)[0].default

    def acceptable_client_address(self, ipaddr):
        # Return true iff the address ipaddr is a valid client address,
        # i.e. belongs to the DHCP range of an access interface
        for interface in self.get_all_of_type(IF_ACCESS):
            if interface.low.compare(ipaddr) <= 0 and \
               interface.high.compare(ipaddr) >= 0:
                return 1
        return 0

    def number_of_masquerading_interfaces(self):
        count = 0
        for interface in self.__ifdict.values():
            if interface.masquerade:
                count = count + 1
        return count

#
# Interface
#

class Interface:
    def __init__(self, name, conf):
        self.name = name
        typename = conf.get(name, "type")
        try:
            self.type = _if_type_dict[typename]
        except KeyError:
            raise ValueError,"interface %s type %s unknown" % (name, typename)
        self.description = conf.get(name, "description")
        self.network = ipaddr.network(conf.get(name, "address"),
                                      conf.get(name, "netmask"),
                                      ipaddr.DEMAND_INTERFACE)

        self.masquerade = 0
        if self.type == IF_DEFAULT:
            # Must have default route
            self.default = ipaddr.ipaddr(conf.get(name, "default"))

        elif self.type == IF_ACCESS:
            # Must have range for DHCP
            self.low = ipaddr.ipaddr(conf.get(name, "low"))
            self.high = ipaddr.ipaddr(conf.get(name, "high"))

            # Masquerading?
            self.masquerade = conf.getbool(name, "masquerade")

        self.fixed = conf.get_default(name, "fixed","")


    def get_network_slash_bits(self):
        # Get network/mask in the 192.168.1.0/24 format
        return "%s/%d" % (self.network.network_str(),
                          self.network.mask.netmask_bits)
    
    def get_fixed_addresses(self):
        # Return a list of tuples (ethernet, ipaddr) of addresses
        # that should be fixed on this interface
        l = []
        for entry in string.split(string.replace(self.fixed, ",", " ")):
            try:
                (eth, ip) = string.split(entry, "=")
                l.append((eth, ipaddr.ipaddr(ip)))
            except:
                pass

        return l

    def __repr__(self):
        return "<Interface %s: type %s at %s/%d>" % (self.name, self.type,
                                                     self.network.ip.ip_str(),
                                                     self.network.mask.netmask_bits)
    

#
# Accessing switches.conf
#

#
# Switches
#

class Switches:
    def __init__(self):
        # Read switches.conf
        self.conf = Config("switches")

        # Create switch objects
        self.__swdict = {}
        for swname in self.conf.sections():
            self.__swdict[swname] = Switch(swname, self.conf)

    def __getitem__(self, swname):
        # Indexing by name
        return self.__swdict[swname]

    def get_all(self):
        # Return a plain list of all switch objects
        return self.__swdict.values()

#
# Switch
#

class Switch:
    def __init__(self, name, conf):
        self.name = name
        self.address = ipaddr.ipaddr(conf.get(name, "address"))
        self.community = conf.get_default(name, "community", "public")
        self.ignore = conf.get_default(name, "ignore", "")
        self.unreach_count = conf.getint_default(name, "unreach_count")
        self.unreach_time = conf.getint_default(name, "unreach_time")
        
    def __repr__(self):
        return "<Switch %s: at %s>" % (self.name, self.address.ip_str())
    
    def ifdescr_on_ignore_list(self, ifdescr):
        ignorelist = string.split(string.replace(self.ignore, ",", " "))
        return ifdescr in ignorelist
    
