# The Netlogon Project
#
# conndb.py - connections database and handling
#
# $Id:#
#
# Copyright (C) 2000,2001 UNIT, Linkping University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import path

import os
import string
import time
import sys

import network
import db
import ipaddr
import log

import conf

#
# Setup database connection
#

db.connect("localhost","netlogon","","netlogon")

#
# Errors
#

class ConnDBError: pass
class ConnDBNoArpInfo(ConnDBError): pass

#
# CLASS CONNECTION
#

CS_OFF = 0
CS_LOGGING_ON = 1
CS_ON = 2

base_query = "SELECT ip, ether, ifname, type, user, UNIX_TIMESTAMP(logged_on) AS logged_on_time_t, lang, swip, swname, swifdescr, swifalias, UNIX_TIMESTAMP(unreach_since) AS unreach_since_time_t, unreach_count, unreach_count_limit, unreach_time_limit, state FROM connections"

class Connection:
    def __init__(self, ip):
        self.__ip = ip
        self.__ether = ""
        self.__ifname = ""
        self.__type = ""
        self.__user = ""
        self.__logged_on = 0
        self.__lang = "sv"

        self.__swip = "" # Stored as string, not ipaddr object
        self.__swname = ""
        self.__swifdescr = ""
        self.__swifalias = ""

        self.__unreach_since = 0
        self.__unreach_count = 0
        self.__unreach_count_limit = 4 # After this many losses: auto-logoff
        self.__unreach_time_limit = 30 # After this time: auto-logoff (anded with above)
        
        self.__state = CS_OFF
        
        self.__from_packets = 0
        self.__from_bytes = 0
        self.__to_packets = 0
        self.__to_bytes = 0

        self.__ifobj = None
        
    def is_logging_on(self):
        return self.__state == CS_LOGGING_ON

    def is_on(self):
        return self.__state == CS_ON

    def get_ether(self):
        return self.__ether

    def get_ifname(self):
        return self.__ifname

    def get_ip(self):
        return self.__ip

    def get_lang(self):
        return self.__lang

    def set_lang(self, lang):
        self.__lang = lang

    def learn_interface_object(self):
        try:
            ifs = conf.Interfaces()
            self.__ifobj = ifs[self.__ifname]
        except:
            pass

    def get_masquerade(self):
        if self.__ifobj:
            return self.__ifobj.masquerade
        else:
            return 0 # Safe default?

    def goto_logging_on(self, ip):
        # Ta reda p interface och ethernet-adress
        try:
            (ifname, ether) = network.get_ifname_and_ether(ip)
        except:
            raise ConnDBNoArpInfo

        # Spara data i objektet
        self.__ifname = ifname
        self.__ether = ether
        self.__state = CS_LOGGING_ON
        self.learn_interface_object()
        
        # Logga
        self.log(event="connection", led_notice=128)

        # Ls ethernet-adressen
        network.fix_ether(ip, ether)

        # Spara data i databasen
        self.save_to_db()

    def log_failed_logging_on(self, cause):
        # Logga
        self.log(event="connection-failure", cause = cause, led_notice = 136)

    def goto_on(self, logon_type, logon_user):
        # Spara data i objektet
        self.__type = logon_type
        self.__user = logon_user
        self.__logged_on = int(time.time())
        self.__state = CS_ON

        # Logga
        self.log(event="logon", led_notice = 129)

        # Lgg till brandvggsregler
        network.add_forwarding_rule(self.__ip, masquerade = self.get_masquerade())

        # Spara data i databasen
        self.save_to_db()

    def log_failed_on(self, logon_type, logon_user, cause):
        # Spara data i objektet
        self.__type = logon_type
        self.__user = logon_user

        # Logga
        self.log(event="logon-failure", cause = cause, led_notice = 137)

    def goto_off(self, cause):
        # Spara data i objektet
        self.__state = CS_OFF

        # Statistik
        try:
            (self.__from_packets,
             self.__from_bytes,
             self.__to_packets,
             self.__to_bytes) = network.get_forwarding_stats(self.__ip)
        except:
            pass
        
        # Logga
        self.log(event="logoff", cause=cause, led_notice = 130)

        # Ta bort brandvggsregler
        network.del_forwarding_rule(self.__ip, masquerade = self.get_masquerade())

        # Ta bort ls p ethernet-adressen
        try:
            network.unfix_ether(self.__ip)
        except:
            pass # If it was not fixed, we don't care

        # Spara data i databasen
        self.save_to_db()

    def log_failed_off(self, cause):
        # Logga
        self.log(event="logoff-failure", cause = cause, led_notice = 138)

    def save_to_db(self):
        # Store state in DB
        # FIXME: What happens if we raise an error here?
        if self.__state == CS_OFF:
            q = "DELETE FROM connections WHERE ip='%s'" % self.__ip.ip_str()
        else:
            q = "REPLACE INTO connections(ip, ether, ifname, type, user, logged_on, lang, swip, swname, swifdescr, swifalias, unreach_since, unreach_count, unreach_count_limit, unreach_time_limit, state) VALUES('%s','%s','%s','%s','%s',FROM_UNIXTIME(%d),'%s','%s','%s','%s','%s',FROM_UNIXTIME(%d),%d,%d,%d,%d)" % (self.__ip.ip_str(), self.__ether, self.__ifname, self.__type, self.__user, self.__logged_on, self.__lang, self.__swip, self.__swname, self.__swifdescr, self.__swifalias, self.__unreach_since, self.__unreach_count, self.__unreach_count_limit, self.__unreach_time_limit, self.__state)

        db.query(q)

    def load_from_db_dict(self, data):
        # Load state from a dictionary
        try:
            self.__ether = data["connections.ether"]
            self.__ifname = data["connections.ifname"]
            self.__type = data["connections.type"]
            self.__user = data["connections.user"]
            self.__logged_on = int(data[".logged_on_time_t"])
            self.__lang = data["connections.lang"]
            self.__swip = data["connections.swip"]
            self.__swname = data["connections.swname"]
            self.__swifdescr = data["connections.swifdescr"]
            self.__swifalias = data["connections.swifalias"]
            self.__unreach_since = int(data[".unreach_since_time_t"])
            self.__unreach_count = data["connections.unreach_count"]
            self.__unreach_count_limit = data["connections.unreach_count_limit"]
            self.__unreach_time_limit = data["connections.unreach_time_limit"]
            self.__state = data["connections.state"]
        except:
            pass

        self.learn_interface_object()

        
    def load_from_db(self):
        # Load state from DB
        try:
            data = db.load_dict_one(base_query + \
                                    " WHERE ip='%s'" % self.__ip.ip_str())
            self.load_from_db_dict(data)
        except:
            pass

    def log(self, event, cause = None, led_notice = None, led_error = None):
        # Send a log message about the state change. Build a list of key=value
        data = []
        data.append(("event", event))
        data.append(("ip", self.__ip.ip_str()))
        if self.__ether <> "": data.append(("ether", self.__ether))
        if self.__ifname <> "": data.append(("interface", self.__ifname))
        if self.__type <> "": data.append(("type", self.__type))
        if self.__user <> "": data.append(("user", self.__user))
        if cause is not None: data.append(("cause", cause))
        if self.__lang <> "": data.append(("lang", self.__lang))

        if self.__logged_on <> 0: data.append(("logged-on", str(self.__logged_on)))

        if (self.__from_packets <> 0 or \
            self.__from_bytes <> 0 or \
            self.__to_packets <> 0 or \
            self.__to_bytes <> 0):
            data.append(("from-packets", str(self.__from_packets)))
            data.append(("from-bytes", str(self.__from_bytes)))
            data.append(("to-packets", str(self.__to_packets)))
            data.append(("to-bytes", str(self.__to_bytes)))

        if self.__swip <> "": data.append(("swip", self.__swip))
        if self.__swname <> "": data.append(("swname", self.__swname))
        if self.__swifdescr <> "": data.append(("swifdescr", self.__swifdescr))
        if self.__swifalias <> "": data.append(("swifalias", self.__swifalias))

        # Format and log
        txt = string.join(map(lambda x: x[0] + "=" + x[1], data), " ")
        log.log(txt, led_notice = led_notice, led_error = led_error)

    # Register that a certain host is at a certain switch and port
    # If the information was changed: return 1 and save to database
    # If the information was not changed: return 0, do not save
    
    def set_location_and_unreach(self, swip, swname, swifdescr, swifalias,
                                 unreach_count, unreach_time):
        if swip <> self.__swip or \
           swname <> self.__swname or \
           swifdescr <> self.__swifdescr or \
           swifalias <> self.__swifalias:
            # Changed (we do not consider unreach parameters in this decision!)
            self.__swip = swip
            self.__swname = swname
            self.__swifdescr = swifdescr
            self.__swifalias = swifalias
            if unreach_count is not None:
                self.__unreach_count_limit = unreach_count
            if unreach_time is not None:
                self.__unreach_time_limit = unreach_time

            q = "UPDATE connections SET swip='%s', swname='%s', swifdescr='%s', swifalias='%s', unreach_count_limit=%d, unreach_time_limit=%d WHERE ether='%s'" % (self.__swip, self.__swname, self.__swifdescr, self.__swifalias, self.__unreach_count_limit, self.__unreach_time_limit, self.__ether)
            try:
                db.query(q)
            except:
                pass

            return 1
        else:
            # Not changed
            return 0
            
            
    # Register reachability for this host
    # Save to database if needed
    
    def set_reachability(self, reachable):
        save = 0

        if reachable:
            # We are reachable
            if self.__unreach_count:
                save = 1 # but we used to be unreachable
            self.__unreach_count = 0
            self.__unreach_since = 0
        else:
            # We are unreachable
            save = 1
            self.__unreach_count = self.__unreach_count + 1
            if self.__unreach_count == 1:
                self.__unreach_since = int(time.time())

        if save:
            q = "UPDATE connections SET unreach_since=FROM_UNIXTIME(%d), unreach_count=%d WHERE ether='%s'" % (self.__unreach_since, self.__unreach_count, self.__ether)
            try:
                db.query(q)
            except:
                pass

    # Check reachability for this host
    # using the information set by "set_reachability"
    
    def is_unreachable(self):
        if self.__unreach_count >= self.__unreach_count_limit and \
           self.__unreach_since < int(time.time()) - self.__unreach_time_limit:
            return 1
        else:
            return 0
       
# Get a connection object (load from database)
def get_connection(ip):
    conn = Connection(ip)
    conn.load_from_db()
    return conn

# Get all logged on connections as a list.
# If parameter also_loggin_on is true, include those too.

def get_all_on(also_loggin_on = 0):
    if also_loggin_on:
        min_state = 1
    else:
        min_state = 2
    datas = db.load_dict_list(base_query + \
                              " WHERE state >= %d" % min_state)
    conns = []
    for data in datas:
        try:
            ip = ipaddr.ipaddr(data["connections.ip"])
            conn = Connection(ip)
            conn.load_from_db_dict(data)
            conns.append(conn)
        except:
            pass
    return conns

# Collect connections statistics
# Return tuple (no_on, no_logging_on)

def collect_connections_statistics():
    no_on = no_logging_on = 0
    try:
        [no_on] = db.load_list_one("SELECT COUNT(*) FROM connections WHERE state=2")
        [no_logging_on] = db.load_list_one("SELECT COUNT(*) FROM connections WHERE state=1")
    except:
        pass
    return (no_on, no_logging_on)
